.. warning::

   Though this MicroPython-based library may be available for use
   in some builds of CircuitPython, it is unsupported and its functionality
   may change in the future, perhaps significantly.
   As CircuitPython continues to develop, it may be changed
   to comply more closely with the corresponding standard Python library.
   You will likely need to change your code later if you rely
   on any non-standard functionality it currently provides.
