print("hello world".rfind("ll"))
print("hello world".rfind("ll", None))
print("hello world".rfind("ll", 1))
print("hello world".rfind("ll", 1, None))
print("hello world".rfind("ll", None, None))
print("hello world".rfind("ll", 1, -1))
print("hello world".rfind("ll", 1, 1))
print("hello world".rfind("ll", 1, 2))
print("hello world".rfind("ll", 1, 3))
print("hello world".rfind("ll", 1, 4))
print("hello world".rfind("ll", 1, 5))
print("hello world".rfind("ll", -100))
print("0000".rfind('0'))
print("0000".rfind('0', 0))
print("0000".rfind('0', 1))
print("0000".rfind('0', 2))
print("0000".rfind('0', 3))
print("0000".rfind('0', 4))
print("0000".rfind('0', 5))
print("0000".rfind('-1', 3))
print("0000".rfind('1', 3))
print("0000".rfind('1', 4))
print("0000".rfind('1', 5))
print("aaaaaaaaaaa".rfind("bbb", 9, 2))
