/* custom CSS for MicroPython docs
 */

.admonition-difference-to-cpython {
    border: 1px solid black;
}

.admonition-difference-to-cpython .admonition-title {
    margin: 4px;
}

/* custom CSS to sticky the ' viewing outdated version'
   warning
*/
.document > .admonition {
    position: sticky;
    top: 0px;
    background-color: salmon;
    z-index: 2;
}

body {
    overflow-x: unset!important;
}

/* override table width restrictions */
@media screen and (min-width: 767px) {

   .wy-table-responsive table td {
      /* !important prevents the common CSS stylesheets from overriding
         this as on RTD they are loaded after this stylesheet */
      white-space: normal !important;
   }

   .wy-table-responsive {
      overflow: visible !important;
   }
}

.strike {
    text-decoration: line-through;
}
