#pragma once

#define PICO_IE_26_29_UNCHANGED_ON_RESET (0)
#define PICO_USE_STACK_GUARDS (0)
#define PICO_ENTER_USB_BOOT_ON_EXIT (0)
#define PICO_USE_OPTIMISTIC_SBRK (0)
#define PICO_NO_HARDWARE (0)
#define PICO_ON_DEVICE (1)
#define PICO_USE_CRT_PRINTF (0)
#define PICO_NO_PRINTF (0)
#define PICO_FLOAT_SUPPORT_ROM_V1 (1)
#define PICO_DOUBLE_SUPPORT_ROM_V1 (1)
#define PICO_STDIO_UART (0)
#define PICO_STDIO_USB (0)
#define PICO_STDIO_SEMIHOSTING (0)
#define PICO_STDIO_IGNORE_NESTED_STDOUT (0)
#define PICO_PRINTF_PICO (0)
#define PICO_PRINTF_NONE (0)
#define PICO_PRINTF_ALWAYS_INCLUDED (1)
