#define MICROPY_HW_BOARD_NAME "Pimoroni Tiny 2040"
#define MICROPY_HW_MCU_NAME "rp2040"

#define CIRCUITPY_RGB_STATUS_INVERTED_PWM
#define CIRCUITPY_RGB_STATUS_R (&pin_GPIO18)
#define CIRCUITPY_RGB_STATUS_G (&pin_GPIO19)
#define CIRCUITPY_RGB_STATUS_B (&pin_GPIO20)

#define MICROPY_HW_USER_SW (&pin_GPIO23)

// These pins are unconnected
#define IGNORE_PIN_GPIO8    1
#define IGNORE_PIN_GPIO9    1
#define IGNORE_PIN_GPIO10   1
#define IGNORE_PIN_GPIO11   1
#define IGNORE_PIN_GPIO12   1
#define IGNORE_PIN_GPIO13   1
#define IGNORE_PIN_GPIO14   1
#define IGNORE_PIN_GPIO15   1
#define IGNORE_PIN_GPIO16   1
#define IGNORE_PIN_GPIO17   1
#define IGNORE_PIN_GPIO21   1
#define IGNORE_PIN_GPIO22   1
#define IGNORE_PIN_GPIO24   1
#define IGNORE_PIN_GPIO25   1
